# Configuration file for the Sphinx documentation builder.
#
# This file only contains a selection of the most common options. For a full
# list see the documentation:
# https://www.sphinx-doc.org/en/master/usage/configuration.html

import re
import sys
import subprocess
from pathlib import Path
from typing import Any, Dict, List

from rocm_docs import ROCmDocs

version_numbers = []
version_file = open("../VERSION", "r")
lines = version_file.readlines()
for line in lines:
    if line[0] == '#':
        continue
    version_numbers.append(line.strip())
version_number = ".".join(version_numbers)
left_nav_title = f"HIP {version_number} Documentation"

# for PDF output on Read the Docs
project = "HIP Documentation"
author = "Advanced Micro Devices, Inc."
copyright = "Copyright (c) 2023 Advanced Micro Devices, Inc. All rights reserved."
version = version_number
release = version_number

external_toc_path = "./sphinx/_toc.yml"

docs_core = ROCmDocs(left_nav_title)
docs_core.run_doxygen(doxygen_root="doxygen", doxygen_path="doxygen/xml")
docs_core.enable_api_reference()
docs_core.setup()

external_projects_current_project = "hip"

for sphinx_var in ROCmDocs.SPHINX_VARS:
    globals()[sphinx_var] = getattr(docs_core, sphinx_var)

# Add the _extensions directory to Python's search path
sys.path.append(str(Path(__file__).parent / 'extension'))

extensions += ["sphinxcontrib.doxylink", "custom_directive"]

cpp_id_attributes = ["__global__", "__device__", "__host__", "__forceinline__", "static"]
cpp_paren_attributes = ["__declspec"]

suppress_warnings = ["etoc.toctree"]

numfig = False

exclude_patterns = [
    "./doxygen/mainpage.md",
    "./understand/glossary.md",
    './how-to/debugging_env.rst',
    "./reference/env_variables"
]

git_url = subprocess.check_output(['git', 'config', '--get', 'remote.origin.url']).strip().decode('ascii')
if git_url.find("git:") != -1:
    html_theme_options = {"repository_url": "https://github.com/ROCm/hip"}



#
# Copyright (C) Advanced Micro Devices, Inc.
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE COPYRIGHT HOLDER(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN
# AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
#

import os
import re
from docutils.parsers.rst import Directive
from docutils.statemachine import StringList

class TableInclude(Directive):
    required_arguments = 1
    optional_arguments = 0
    final_argument_whitespace = True
    option_spec = {
        'table': str
    }

    def run(self):
        # Get the file path from the first argument
        file_path = self.arguments[0]

        # Get the environment to resolve the full path
        env = self.state.document.settings.env
        src_dir = os.path.abspath(env.srcdir)
        full_file_path = os.path.join(src_dir, file_path)

        # Check if the file exists
        if not os.path.exists(full_file_path):
            raise self.error(f"RST file {full_file_path} does not exist.")

        # Read the entire file content
        with open(full_file_path, 'r', encoding='utf-8') as f:
            content = f.read()

        # Find all tables with named targets
        table_pattern = r'(?:^\.\.\ _(.+?):\n)(.. list-table::.*?(?:\n\s*\*\s*-.*?)+)(?=\n\n|\Z)'
        table_matches = list(re.finditer(table_pattern, content, re.MULTILINE | re.DOTALL))

        # Get the specific table name from options
        table_name = self.options.get('table')

        # If no table specified, merge compatible tables
        if not table_name:
            raise self.error("The ':table:' option is required to specify which table to include.")

        # Find the specific table
        matching_tables = [
            match for match in table_matches
            if match.group(1).strip() == table_name
        ]

        if not matching_tables:
            raise self.error(f"Table '{table_name}' not found in {full_file_path}")

        # Extract the matched table content
        table_content = matching_tables[0].group(2)

        # Insert the table content into the current document
        self.state_machine.insert_input(table_content.splitlines(), full_file_path)
        return []

def setup(app):
    app.add_directive('include-table', TableInclude)



#
# Copyright (C) Advanced Micro Devices, Inc.
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE COPYRIGHT HOLDER(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN
# AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
#

import urllib.request

urllib.request.urlretrieve("https://raw.githubusercontent.com/ROCm/rocm-examples/refs/heads/develop/HIP-Basic/opengl_interop/main.hip", "docs/tools/example_codes/opengl_interop.hip")
urllib.request.urlretrieve("https://raw.githubusercontent.com/ROCm/rocm-examples/refs/heads/develop/HIP-Basic/vulkan_interop/main.hip", "docs/tools/example_codes/external_interop.hip")



import os, re, sys
from typing import List, Optional


def is_checkbox(line: str) -> bool:
    return bool(re.match(r"^\s*-\s*\[[ xX]\]\s*.+", line))


def is_checked(line: str) -> bool:
    return bool(re.match(r"^\s*-\s*\[[xX]\]\s*.+", line))


def is_comment(line: str) -> bool:
    return bool(re.match(r"^\s*<!--.*-->\s*$", line))


def text_clean(lines: List[str]) -> str:
    text = [line for line in lines if not is_comment(line)]
    return "".join("".join(text).strip().split())


def validate_section(section_name: str, lines: List[str]) -> Optional[str]:
    has_checkboxes = any(is_checkbox(line) for line in lines)
    if has_checkboxes:
        if not any(is_checked(line) for line in lines):
            return f"Section {section_name} is a checklist without selections"
        return None
    if not text_clean(lines):
        return f"Section {section_name} is empty text section"
    return None


def check_description(description: str) -> List[str]:
    if not description:
        # pull_request_template is not merged yet, so treat as valid for now
        return []
        # return ["PR description is empty"]

    sections = []
    current_section = None
    current_lines = []
    errors = []

    for line in description.splitlines():
        header_match = re.match(r"^\s*##\s*(.+?)\s*$", line)
        if header_match:
            if current_section:
                sections.append((current_section, current_lines))
            current_section = header_match.group(1)
            current_lines = []
        elif current_section:
            current_lines.append(line)

    if current_section:
        sections.append((current_section, current_lines))

    if not sections:
        return ["No sections available, template is empty"]

    for section_name, section_lines in sections:
        error = validate_section(section_name, section_lines)
        if error:
            errors.append(error)

    return errors


if __name__ == "__main__":
    pr_description = os.getenv("PR_DESCRIPTION", "")

    errors = check_description(pr_description)
    if not errors:
        print("All good")
        exit(0)
    print("\n".join(errors))
    exit(1)
